@@ -90,6 +90,8 @@ static struct directory_entry directory_table[] = {
 	{ "fi",		"ISO-8859-1",	"ISO-8859-1"		}, /* Finnish */
 	{ "fr",		"ISO-8859-1",	"ISO-8859-1"		}, /* French */
 	{ "ga",		"ISO-8859-1",	"ISO-8859-1"		}, /* Irish */
+	{ "gl",		"ISO-8859-1",	"ISO-8859-1"		}, /* Galician */
+	{ "id",		"ISO-8859-1",	"ISO-8859-1"		}, /* Indonesian */
 	{ "is",		"ISO-8859-1",	"ISO-8859-1"		}, /* Icelandic */
 	{ "it",		"ISO-8859-1",	"ISO-8859-1"		}, /* Italian */
 	{ "nl",		"ISO-8859-1",	"ISO-8859-1"		}, /* Dutch */
@@ -102,11 +104,14 @@ static struct directory_entry directory_table[] = {
 	 * ascii8 and nippon devices.
 	 */
 	{ "cs",		"ISO-8859-2",	"ISO-8859-2"		}, /* Czech */
+	{ "hr",		"ISO-8859-2",	"ISO-8859-2"		}, /* Croatian */
 	{ "hu",		"ISO-8859-2",	"ISO-8859-2"		}, /* Hungarian */
 	{ "ja",		"EUC-JP",	"EUC-JP"		}, /* Japanese */
 	{ "ko",		"EUC-KR",	"EUC-KR"		}, /* Korean */
 	{ "pl",		"ISO-8859-2",	"ISO-8859-2"		}, /* Polish */
 	{ "ru",		"KOI8-R",	"KOI8-R"		}, /* Russian */
+	{ "sk",		"ISO-8859-2",	"ISO-8859-2"		}, /* Slovak */
+	{ "tr",		"ISO-8859-9",	"ISO-8859-9"		}, /* Turkish */
 #endif /* MULTIBYTE_GROFF */
 
 	{ NULL,		NULL,		"NULL"			}
@@ -126,6 +131,7 @@ struct charset_entry {
 static struct charset_entry charset_table[] = {
 	{ "ANSI_X3.4-1968",	"ascii"		},
 	{ "ISO-8859-1",		"latin1"	},
+	{ "ISO-8859-15",    	"latin1"	},
 	{ "UTF-8",		"utf8"		},
 
 #ifdef MULTIBYTE_GROFF
@@ -200,8 +206,10 @@ static const char *fallback_less_charset = "iso8859";
 /* Return the assumed encoding of the source man page, based on the
  * directory in which it was found. The caller should attempt to recode from
  * this to whatever encoding is expected by groff.
+ *
+ * The caller should free the returned string when it is finished with it.
  */
-const char *get_page_encoding (const char *lang)
+char *get_page_encoding (const char *lang)
 {
 	const struct directory_entry *entry;
 	const char *dot;
@@ -210,7 +218,7 @@ const char *get_page_encoding (const char *lang)
 		/* Guess based on the locale. */
 		lang = setlocale (LC_MESSAGES, NULL);
 		if (!lang)
-			return fallback_source_encoding;
+			return xstrdup (fallback_source_encoding);
 	}
 
 	dot = strchr (lang, '.');
@@ -235,14 +243,13 @@ const char *get_page_encoding (const char *lang)
 		 * grounds that it's obviously wrong, and petition to have
 		 * it changed.
 		 */
-		/* TODO: I should probably drop any ,<version> components. */
-		return dot + 1;
+		return xstrndup (dot + 1, strcspn (dot + 1, ",@"));
 
 	for (entry = directory_table; entry->lang_dir; ++entry)
 		if (STRNEQ (entry->lang_dir, lang, strlen (entry->lang_dir)))
-			return entry->source_encoding;
+			return xstrdup (entry->source_encoding);
 
-	return fallback_source_encoding;
+	return xstrdup (fallback_source_encoding);
 }
 
 /* Return the canonical encoding for source man pages in the specified
@@ -298,8 +305,10 @@ const char *get_source_encoding (const char *lang)
 /* Return the standard output encoding for the source man page, based on the
  * directory in which it was found. This should only be used to determine
  * whether a cat page can be saved.
+ *
+ * The caller should free the returned string when it is finished with it.
  */
-const char *get_standard_output_encoding (const char *lang)
+char *get_standard_output_encoding (const char *lang)
 {
 	const struct directory_entry *entry;
 	const char *dot;
@@ -317,12 +326,11 @@ const char *get_standard_output_encoding (const char *lang)
 		 * man directory including an explicit character set, so the
 		 * pages it contains should have that encoding.
 		 */
-		/* TODO: I should probably drop any ,<version> components. */
-		return dot + 1;
+		return xstrndup (dot + 1, strcspn (dot + 1, ",@"));
 
 	for (entry = directory_table; entry->lang_dir; ++entry)
 		if (STRNEQ (entry->lang_dir, lang, strlen (entry->lang_dir)))
-			return entry->standard_output_encoding;
+			return xstrdup (entry->standard_output_encoding);
 
 	return NULL;
 }
