@@ -186,7 +186,7 @@ static char *ult_softlink (const char *fullpath)
 static char *test_for_include (const char *buffer)
 {
 	/* strip out any leading whitespace (if any) */
-	while (isspace ((int) *buffer))
+	while (CTYPE (isspace, *buffer))
 		buffer++;
 
 	/* see if the `command' is a .so */
@@ -195,7 +195,7 @@ static char *test_for_include (const char *buffer)
 
 		/* strip out any whitespace between the command and 
 		   it's argumant */
-		while (isspace ((int) *buffer))
+		while (CTYPE (isspace, *buffer))
 			buffer++;
 
 		/* If .so's argument is an absolute filename, it could be
@@ -209,7 +209,7 @@ static char *test_for_include (const char *buffer)
 		 * ultimate source file */
 		if (*buffer != '/') {
 			const char *end = buffer;
-			while (*end && !isspace (*end))
+			while (*end && !CTYPE (isspace, *end))
 				++end;
 			return xstrndup (buffer, end - buffer);
 		}
@@ -303,9 +303,12 @@ const char *ult_src (const char *name, const char *path,
 
 		/* if we are handed the name of a compressed file, remove
 		   the compression extension? */
-		comp = comp_info (basename);
-		if (comp)
-			*comp->file = '\0';
+		comp = comp_info (basename, 1);
+		if (comp) {
+			free (basename);
+			basename = comp->stem;
+			comp->stem = NULL; /* steal memory */
+		}
 
 		/* if the open fails, try looking for compressed */
 		fp = fopen (basename, "r");
@@ -314,8 +317,8 @@ const char *ult_src (const char *name, const char *path,
 
 			comp = comp_file (basename);
 			if (comp) {
-				filename = decompress (comp->file, comp);
-				free (comp->file);
+				filename = decompress (comp->stem, comp);
+				free (comp->stem);
 				if (!filename)
 					return NULL;
 				basename = strappend (basename, ".", comp->ext,
